<!DOCTYPE html>
<html>
<head lang="en">
    <meta charset="UTF-8">
    <title>{{ tests_sum.name }}</title>
    <link href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.2/css/bootstrap.min.css" rel="stylesheet">
    <link href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.2/css/bootstrap-theme.min.css" rel="stylesheet">
    <style type="text/css">
		h3 {
            font-weight: bold;
        }
        caption {
            font-size: 1.5em;
        }
        table {
            table-layout: fixed;
            word-wrap: break-word;
            overflow: auto;
        }
    </style>
</head>
<body>
<div class="container-fluid">
    <h3>{{ tests_sum.name|e|safe }}</h3>
    <p>
        Generated by {{ app }}-{{ version }} at {{ date_time }}<br/>
        {{ description|e|nl2br|safe }}
    </p>
</div>

<div class="container-fluid">
    <table class="table table-hover">
        <caption>Summary</caption>
        <thead>
        <tr class="info">
            <th>Duration</th>
            <th>Total</th>
            <th>Pass</th>
            <th>Skipped</th>
            <th>Error</th>
            <th>Fail</th>
            <th>Rate</th>
        </tr>
        </thead>
        <tbody>
        <tr class="{{ tests_sum.category }}">
            <td>{{ tests_sum.duration }}s</td>
            <td>{{ tests_sum.run }}</td>
            <td>{{ tests_sum.passed }}</td>
            <td>{{ tests_sum.skipped }}</td>
            <td>{{ tests_sum.error }}</td>
            <td>{{ tests_sum.fail }}</td>
            <td>{{ tests_sum.rate }}</td>
        </tr>
        </tbody>
    </table>

    <table class="table table-hover table-bordered">
        <caption>Test Suites</caption>
        <thead>
        <tr class="info">
            <th style="width: 30%">Suite</th>
            <th style="width: 10%">Duration</th>
            <th style="width: 10%">Total</th>
            <th style="width: 10%">Pass</th>
            <th style="width: 10%">Skipped</th>
            <th style="width: 10%">Error</th>
            <th style="width: 10%">Fail</th>
            <th style="width: 10%">Rate</th>
        </tr>
        </thead>
        <tbody>
        {% for suite_sum in tests_sum.suite_sum_list %}
        <tr class="{{ suite_sum.category }}">
            <td>{{ suite_sum.name }}</td>
            <td>{{ suite_sum.duration }}s</td>
            <td>{{ suite_sum.run }}</td>
            <td>{{ suite_sum.passed }}</td>
            <td>{{ suite_sum.skipped }}</td>
            <td>{{ suite_sum.error }}</td>
            <td>{{ suite_sum.fail }}</td>
            <td>{{ suite_sum.rate }}</td>
        </tr>
        {% endfor %}
        </tbody>
    </table>
</div>
</body>
</html>